		/*//////////   SCRIPT:   LightSelectScript     //////////*/
		/*//////////   WRITTEN BY:   ShyWhy	      //////////*/
		
		
		
		--Version 1.3
			--Added VRayIES, VRaySun and VRayAmbientLight
		
		--Version 1.2
			--Added LightSelect As 1 Checkbox + functions
		
		--Version 1.1
			--Rewrote script
		
		--Version 1.0
			--Core functions
			
			
			
			
try(closerolloutfloater MainFloater)
catch()

--MAIN DIALOG
Rollout Menu01 "Render Elements"
(		
	
	group "Lights"
	(
		radiobuttons light_sources labels:#("All lights","Current selection")  pos:[80,20] height:45
		checkbox only_on "Only On" pos:[80,60] checked:false
		checkbox as_one "As 1" pos:[80,75] checked:false
	)
	
	group "Modes"
	(
		checkbox direct_illumination "Direct Illumination" pos:[80,130] checked:false
		checkbox direct_raw "Direct Raw" pos:[80,150] checked:false
		checkbox direct_diffuse "Direct Diffuse" pos:[80,170] checked:false
		checkbox direct_specular "Direct Specular" pos:[80,190] checked:false
		checkbox full "Full" pos:[80,210] checked:true
	)
	
	group "Options"
	(
		checkbox clear_list "Clear list before adding" pos:[50,260] checked:false
		checkbox remove_lightSelect "Remove all Light Selects" pos:[50,280] checked:false
		checkbox group_instances "Add instances as 1" pos:[50,300] checked:false enabled:true
		checkbox accept_duplicates "Duplication allowed" pos:[50,320] checked:false
	)
	
	button btn_Apply "Start" pos:[60,370] width:75 height:20
	
	
	--Check if there is already an instance of the light source
	fn isFirstInstance &light_element &lights_array = 
	(
		InstanceMgr.GetInstances light_element &instances
		
		
		for i = 1 to lights_array.count do
		(
			for j = 1 to instances.count do
			(
					if instances[j] == lights_array[i] then
					(
							return false
					)
			)
		)
		return true
	)
	
	
	--Adds light source
	fn AddLight &light_element &lights_array =
	(
		append lights_array light_element
	)
	
	
	--Check for instances if user selected this option
	fn InstanceOptionCheck &light_element &lights_array = 
	(
		if group_instances.checked and group_instances.enabled and isFirstInstance light_element lights_array then
		(
			AddLight light_element lights_array
		)	
		else if group_instances.checked == false or group_instances.enabled == false then
		(
			AddLight light_element lights_array
		)
	)
	
	
	--Ignore the disabled light sources
	fn VRayOnlyOnCheck &light_element &lights_array = 
	(
		local lightClass = classof light_element.baseObject
		
		
		if lightClass == VRayLight or lightClass== VRaySun or lightClass == VRayIES or lightClass == VRayAmbientLight  then
		(
			if only_on.checked and light_element.on then
			(
				InstanceOptionCheck light_element lights_array
			)
			else if only_on.checked == false then
			(
				InstanceOptionCheck light_element lights_array
			)
		)
	)
	
	
	--Check the selection if user selected this option
	fn SelectionCheck &light_element &lights_array =
	(
		if light_sources.state == 1 then
		(
			VRayOnlyOnCheck light_element lights_array
		)
		else if light_sources.state == 2 and light_element.isSelected then
		(
			VRayOnlyOnCheck light_element lights_array
		)
	)
	
	
	--Select only one if "as one-option" is selected
	fn SelectOnlyOne &lights_array =
	(
		local gotOne = false
			
		for light_element in lights do
		(
			if gotOne == false then
			(
				local lightClass = classof light_element.baseObject
				
				if lightClass == VRayLight or lightClass== VRaySun or lightClass == VRayIES or lightClass == VRayAmbientLight then
				(
					if light_sources.state == 2 and light_element.isSelected then
					(
						if only_on.checked and light_element.on then
						(
							AddLight light_element lights_array
							gotOne = true
						)
						else if only_on.checked == false then
						(
							AddLight light_element lights_array
							gotOne = true
						)
					)
					else if light_sources.state == 1 then
					(
						if only_on.checked and light_element.on then
						(
							AddLight light_element lights_array
							gotOne = true
						)
						else if only_on.checked == false then
						(
							AddLight light_element lights_array
							gotOne = true
						)
					)
				)
			)
		)	
	)
	
	--Select the requiered light sources
	fn GetLights &lights_array = 
	(
		if as_one.checked then
		(
			SelectOnlyOne lights_array
		)
		else
		(
			for light_element in lights do
			(
				SelectionCheck light_element lights_array
			)
		)
	)
	
	
	--Check if there is already a LightSelect with this light source
	fn CheckDuplicateOption &re &name =
	(
		if accept_duplicates.checked then
		(
				return true
		)	
		else
		(
			local elementsCounter = re.NumRenderElements()
			local element
			local classOf_element
			
			for i = 1 to elementsCounter do
			(
				element = re.GetRenderElement (elementsCounter - i)
				classOf_element = classof element
				if classOf_element == VRayLightSelect then
				(
					if element.elementName == name then
					(
						return false
					)
				)				
			)
			return true
		)
	)
	
	
	--Adds VRayLightSelect + properties
	fn CreateLightSelect &re &lightSelect_Lights &mode &name =
	(
		if CheckDuplicateOption re name == true then
		(
			render_Element = VRayLightSelect()
			
			render_Element.lights = lightSelect_Lights
			render_Element.mode = mode		
			render_Element.elementName = name
			re.AddRenderElement(render_Element)
		)
	)
	
	
	--Check what modes were activated by the user
	fn CreateLightSelectModes &re &lightSelect_Lights &light_elementName=
	(
		if direct_illumination.checked then
		(
			CreateLightSelect re lightSelect_Lights  0 (light_elementName + "_Direct_Illumination")
		)
		if direct_raw.checked then
		(
			CreateLightSelect re lightSelect_Lights  1 (light_elementName + "_Direct_Raw")
		)
		if direct_diffuse.checked then
		(
			CreateLightSelect re lightSelect_Lights  2 (light_elementName + "_Direct_Diffuse")
		)
		if direct_specular.checked then
		(
			CreateLightSelect re lightSelect_Lights  3 (light_elementName + "_Direct_Specular")
		)
		if full.checked then
		(	
			CreateLightSelect re lightSelect_Lights  4 (light_elementName + "_Full")
		)
		if direct_illumination.checked == false and direct_raw == false and direct_diffuse == false and direct_specular == false and full.checked == false then
		(
				messageBox "No mode for VRayLightSelect selected!"
		)			
	)
	
	
	--Add Light to VRayLightSelect
	fn AddLightToLightSelect &lightSelect_Lights &instance =
	(
		append lightSelect_Lights instance
	)
	
	
	--Check if other instances are also selected if this option is activated
	fn CheckInstanceSelection &lightSelect_Lights &instance =
	(
			if light_sources.state == 2 and instance.isSelected then
			(
				AddLightToLightSelect lightSelect_Lights instance
			)
			else if light_sources.state == 1 then
			(
				AddLightToLightSelect lightSelect_Lights instance
			)
	)
	
	
	fn SelectAllOtherLights &lightSelect_Lights &light_element =
	(
		for otherLight in lights do
		(
			local lightClass = classof otherLight.baseObject
				
			if otherLight != light_element and (lightClass == VRayLight or lightClass== VRaySun or lightClass == VRayIES or lightClass == VRayAmbientLight) then
			(
				if light_sources.state == 2 and otherLight.isSelected then
				(
					if only_on.checked and otherLight.on then
					(
						AddLightToLightSelect lightSelect_Lights otherLight
					)
					else if only_on.checked == false then
					(
						AddLightToLightSelect lightSelect_Lights otherLight
					)
				)
				else if light_sources.state == 1 then
				(
					if only_on.checked and otherLight.on then
					(
						AddLightToLightSelect lightSelect_Lights otherLight
					)
					else if only_on.checked == false then
					(
						AddLightToLightSelect lightSelect_Lights otherLight
					)	
				)
			)
		)
	)
	
	--Check if all instances go into one VRayLightSelect
	fn CheckInstancesAsOne &re &light_element =
	(
		lightSelect_Lights = #()
		if as_one.checked then
		(
			SelectAllOtherLights lightSelect_Lights light_element
		)
		else(
			if group_instances.checked and group_instances.enabled then
			(
				InstanceMgr.GetInstances light_element &instances
				for instance in instances do
				(
					if light_element != instance then
					(
						CheckInstanceSelection lightSelect_Lights instance
					)
				)
			)
		)
		AddLightToLightSelect lightSelect_Lights light_element
		
		
		CreateLightSelectModes re lightSelect_Lights light_element.name
	)
	
	
	--Main
	fn main = 
	(
		--Get Render Element Manager
		local re = maxOps.GetCurRenderElementMgr()
		
		--Holding the VRayLights
		local lights_array = #()
		
		--Select Lights
		if lights.count > 0 then
		(
				GetLights lights_array
		)
		else
		(
				messageBox "No lights in the scene"
		)
		
		--Remove All Render Elements
		if clear_list.checked then
		(
			re.RemoveAllRenderElements()
		)
		
		--Remove all VRayLightSelects
		if remove_lightSelect.checked == true then
		(			
			elementsCounter = re.NumRenderElements()
			
			for i = 1 to elementsCounter do
			(
				local element = re.GetRenderElement (elementsCounter - i)
				local class_element = classof element
				
				if class_element == VRayLightSelect then
				(
					re.RemoveRenderElement element
				)
			)
		)
		
		
		--Create VRayLightSelects
		if lights_array.count > 0 then
		(
			for light_element in lights_array do
			(
				CheckInstancesAsOne re light_element
			)
		)
		else
		(
			messageBox "No lights in the Scene or no lights selected!"
		)
		free lights_array
	)
	
	fn UpdateAsOne = 
	(
			if as_one.checked then
			(
					group_instances.enabled = false
			)
			else
			(
					group_instances.enabled = true
			)
	)
	
	
	
	on btn_Apply pressed do main ()
	on as_one changed state do UpdateAsOne()
)

MainFloater = NewRolloutFloater "Render Elements" 208 420
addRollout Menu01 MainFloater
